import React, { useEffect, useRef, useState } from "react";
import { asElement } from "../lib/pdfjs-dom";
import "../style/MouseSelection.css";
const getBoundingRect = (start, end) => {
    return {
        left: Math.min(end.x, start.x),
        top: Math.min(end.y, start.y),
        width: Math.abs(end.x - start.x),
        height: Math.abs(end.y - start.y),
    };
};
// It's impossible to screenshot or open a context menu on a rect with 0 area
// So we need to prevent their render.
const shouldRender = (boundingRect) => {
    return boundingRect.width >= 1 && boundingRect.height >= 1;
};
/**
 * A component that enables the creation of rectangular and interactive
 * mouse selections within a given container. NOTE: This does not disable
 * selection in whatever container the component is placed in. That must be handled
 * through the onDragStart and onDragEnd events.
 */
const MouseSelection = ({ onSelection, onDragStart, onDragEnd, shouldStart, onChange, style, }) => {
    const [start, setStart] = useState(null);
    const [end, setEnd] = useState(null);
    const [locked, setLocked] = useState(false);
    const rootRef = useRef(null);
    // Needed in order to grab the page info of a mouse selection
    const startTargetRef = useRef(null);
    const reset = () => {
        // onDragEnd(); Not sure why this was ever here??
        setStart(null);
        setEnd(null);
        setLocked(false);
    };
    useEffect(() => {
        const isVisible = Boolean(start && end);
        onChange(isVisible);
    }, [start, end]);
    useEffect(() => {
        if (!rootRef.current)
            return;
        const container = asElement(rootRef.current.parentElement);
        let containerBoundingRect = null;
        const containerCoords = (pageX, pageY) => {
            containerBoundingRect ||= container.getBoundingClientRect();
            return {
                x: pageX - containerBoundingRect.left + container.scrollLeft,
                y: pageY -
                    containerBoundingRect.top +
                    container.scrollTop -
                    window.scrollY,
            };
        };
        const handleMouseMove = (event) => {
            if (!start || locked)
                return;
            setEnd(containerCoords(event.pageX, event.pageY));
        };
        const handleMouseDown = (event) => {
            if (!shouldStart(event)) {
                reset();
                return;
            }
            startTargetRef.current = asElement(event.target);
            onDragStart();
            setStart(containerCoords(event.pageX, event.pageY));
            setEnd(null);
            setLocked(false);
        };
        const handleMouseUp = (event) => {
            if (!start)
                return;
            const newEnd = containerCoords(event.pageX, event.pageY);
            const boundingRect = getBoundingRect(start, newEnd);
            if (!container.contains(asElement(event.target)) ||
                !shouldRender(boundingRect)) {
                reset();
                return;
            }
            setEnd(newEnd);
            setLocked(true);
            if (start && end && startTargetRef.current) {
                onSelection(startTargetRef.current, boundingRect, reset);
                onDragEnd();
            }
        };
        container.addEventListener("mousemove", handleMouseMove);
        container.addEventListener("mousedown", handleMouseDown);
        const { ownerDocument: doc } = container;
        if (doc.body)
            doc.body.addEventListener("mouseup", handleMouseUp);
        return () => {
            container.removeEventListener("mousemove", handleMouseMove);
            container.removeEventListener("mousedown", handleMouseDown);
            doc.body.removeEventListener("mouseup", handleMouseUp);
        };
    }, [start, end]);
    return (React.createElement("div", { className: "MouseSelection-container", ref: rootRef }, start && end && (React.createElement("div", { className: "MouseSelection", style: { ...getBoundingRect(start, end), ...style } }))));
};
export default MouseSelection;
//# sourceMappingURL=MouseSelection.js.map