import "pdfjs-dist/web/pdf_viewer.css";
import "../style/PdfHighlighter.css";
import "../style/pdf_viewer.css";
import debounce from "lodash.debounce";
import { EventBus, NullL10n, PDFLinkService, PDFViewer, } from "pdfjs-dist/legacy/web/pdf_viewer";
import React, { useLayoutEffect, useRef, useState, } from "react";
import { createRoot } from "react-dom/client";
import { usePdfDocument } from "../contexts/PdfLoaderContext";
import { SelectionContext } from "../contexts/SelectionContext";
import { TipViewerContext } from "../contexts/TipContext";
import { scaledToViewport, viewportPositionToScaled } from "../lib/coordinates";
import { disableTextSelection } from "../lib/disable-text-selection";
import getBoundingRect from "../lib/get-bounding-rect";
import getClientRects from "../lib/get-client-rects";
import groupHighlightsByPage from "../lib/group-highlights-by-page";
import { asElement, findOrCreateContainerLayer, getPagesFromRange, getWindow, isHTMLElement, } from "../lib/pdfjs-dom";
import HighlightLayer from "./HighlightLayer";
import MouseSelectionRenderer from "./MouseSelectionRenderer";
import TipRenderer from "./TipRenderer";
const SCROLL_MARGIN = 10;
const TIP_WAIT = 250; // Debounce wait time in milliseconds for a selection changing and a tip being displayed
const DEFAULT_SCALE_VALUE = "auto";
const DEFAULT_TEXT_SELECTION_COLOR = "rgba(153,193,218,255)";
/**
 * This is a large-scale PDF viewer component designed to facilitate highlighting.
 * It should be used as a child to a PdfLoader to ensure proper document loading.
 * This does not handle rendering of highlights, but it can provide a HighlightContext
 * to a HighlightRenderer for each highlight per page per document with capabilities
 * to generate tips. However, this does handle text selection and (optionally) area selection.
 */
const PdfHighlighter = ({ highlights, onScrollAway, scrollRef, tipViewerUtilsRef, pdfViewerRef, pdfScaleValue = DEFAULT_SCALE_VALUE, onSelectionFinished, selectionTip, enableAreaSelection, mouseSelectionStyle, children, textSelectionColor = DEFAULT_TEXT_SELECTION_COLOR, style, }) => {
    const containerNodeRef = useRef(null);
    const pdfDocument = usePdfDocument();
    // These are all refs because
    // 1. We need to use their updated states immediately
    // 2. HighlightLayers are manually rendered per page and thus unaffected by state
    const highlightBindingsRef = useRef({}); // Reference to highlight bindings per page
    const ghostHighlightRef = useRef(null); // Reference to in-progress/temporary highlight
    const textSelectionRef = useRef({
        selection: null,
        range: null,
        text: "",
    });
    const scrolledToHighlightIdRef = useRef(null); // Reference to the ID of the highlight autoscrolled to
    const isAreaSelectionInProgressRef = useRef(false);
    const isEditInProgressRef = useRef(false);
    const [tip, setTip] = useState(null);
    const [isViewerReady, setIsViewerReady] = useState(false);
    // These should only change when a document loads/unloads
    const eventBusRef = useRef(new EventBus());
    const linkServiceRef = useRef(new PDFLinkService({
        eventBus: eventBusRef.current,
        externalLinkTarget: 2,
    }));
    const resizeObserverRef = useRef(null);
    const viewerRef = useRef(null);
    const findOrCreateHighlightLayer = (textLayer) => {
        return findOrCreateContainerLayer(textLayer, "PdfHighlighter__highlight-layer");
    };
    // Init listeners
    useLayoutEffect(() => {
        resizeObserverRef.current = new ResizeObserver(handleScaleValue);
        const doc = containerNodeRef.current?.ownerDocument;
        if (!doc || !containerNodeRef.current)
            return;
        resizeObserverRef.current.observe(containerNodeRef.current);
        eventBusRef.current.on("textlayerrendered", renderHighlightLayers);
        eventBusRef.current.on("pagesinit", handleScaleValue);
        doc.addEventListener("selectionchange", handleSelectionChange);
        doc.addEventListener("keydown", handleKeyDown);
        renderHighlightLayers();
        return () => {
            eventBusRef.current.off("pagesinit", handleScaleValue);
            eventBusRef.current.off("textlayerrendered", renderHighlightLayers);
            doc.removeEventListener("selectionchange", handleSelectionChange);
            doc.removeEventListener("keydown", handleKeyDown);
            resizeObserverRef.current?.disconnect();
        };
    }, [selectionTip, highlights, onSelectionFinished]);
    // Initialise PDF Viewer
    useLayoutEffect(() => {
        const doc = containerNodeRef.current?.ownerDocument;
        if (!doc || !containerNodeRef.current)
            return;
        viewerRef.current =
            viewerRef.current ||
                new PDFViewer({
                    container: containerNodeRef.current,
                    eventBus: eventBusRef.current,
                    textLayerMode: 2, // Needed to prevent flickering in Chrome. Known issue with PDF.js
                    removePageBorders: true,
                    linkService: linkServiceRef.current,
                    l10n: NullL10n, // No localisation
                });
        linkServiceRef.current.setDocument(pdfDocument);
        linkServiceRef.current.setViewer(viewerRef.current);
        viewerRef.current.setDocument(pdfDocument);
        setIsViewerReady(true);
    }, []);
    const isTextSelectionEmpty = () => {
        return (!textSelectionRef.current ||
            textSelectionRef.current.selection?.isCollapsed ||
            !textSelectionRef.current.range ||
            !textSelectionRef.current.text);
    };
    const isSelectionInProgress = () => {
        return (!isTextSelectionEmpty() ||
            Boolean(ghostHighlightRef.current) ||
            isAreaSelectionInProgressRef.current ||
            isEditInProgressRef.current);
    };
    const removeGhostHighlight = () => {
        ghostHighlightRef.current = null;
        renderHighlightLayers();
    };
    const scrollTo = (highlight) => {
        const { boundingRect, usePdfCoordinates } = highlight.position;
        const pageNumber = boundingRect.pageNumber;
        // Remove scroll listener in case user auto-scrolls in succession.
        onScrollAway &&
            viewerRef.current.container.removeEventListener("scroll", handleScroll);
        const pageViewport = viewerRef.current.getPageView(pageNumber - 1).viewport;
        viewerRef.current.scrollPageIntoView({
            pageNumber,
            destArray: [
                null, // null since we pass pageNumber already as an arg
                { name: "XYZ" },
                ...pageViewport.convertToPdfPoint(0, // Default x coord
                scaledToViewport(boundingRect, pageViewport, usePdfCoordinates).top -
                    SCROLL_MARGIN),
                0, // Default z coord
            ],
        });
        scrolledToHighlightIdRef.current = highlight.id;
        renderHighlightLayers();
        // wait for scrolling to finish
        setTimeout(() => {
            onScrollAway &&
                viewerRef.current.container.addEventListener("scroll", handleScroll, {
                    once: true,
                });
        }, 100);
    };
    const handleScroll = () => {
        onScrollAway();
        scrolledToHighlightIdRef.current = null;
        renderHighlightLayers();
    };
    const handleSelectionChange = () => {
        const container = containerNodeRef.current;
        const selection = getWindow(container).getSelection();
        if (!container || !selection || selection.isCollapsed)
            return;
        const range = selection.rangeCount > 0 ? selection.getRangeAt(0) : null;
        if (!range ||
            !container.contains(range.commonAncestorContainer) // Check the selected text is in the document, not the tip
        ) {
            return;
        }
        textSelectionRef.current = {
            selection,
            range,
            text: selection.toString().split("\n").join(" "), // Make all line breaks spaces
        };
        debouncedHandleAfterSelection();
    };
    const handleAfterSelection = () => {
        const { selection, range, text } = textSelectionRef.current;
        if (!range || selection?.isCollapsed) {
            return;
        }
        const pages = getPagesFromRange(range);
        if (!pages || pages.length === 0) {
            return;
        }
        const rects = getClientRects(range, pages);
        if (rects.length === 0) {
            return;
        }
        const boundingRect = getBoundingRect(rects);
        const viewportPosition = {
            boundingRect,
            rects,
        };
        const content = { text };
        const scaledPosition = viewportPositionToScaled(viewportPosition, viewerRef.current);
        const selectionUtils = {
            selectionPosition: scaledPosition,
            selectionContent: content,
            removeGhostHighlight,
            makeGhostHighlight: () => {
                ghostHighlightRef.current = {
                    content: content,
                    position: scaledPosition,
                };
                renderHighlightLayers();
            },
        };
        if (onSelectionFinished)
            onSelectionFinished(selectionUtils);
        if (selectionTip) {
            const newTip = {
                position: viewportPosition,
                content: (React.createElement(SelectionContext.Provider, { value: selectionUtils }, selectionTip)),
            };
            setTip(newTip);
        }
    };
    const debouncedHandleAfterSelection = debounce(handleAfterSelection, TIP_WAIT);
    const handleMouseDown = (event) => {
        if (!isHTMLElement(event.target) ||
            asElement(event.target).closest(".PdfHighlighter__tip-container") // Ignore selections on tip container
        ) {
            return;
        }
        setTip(null);
        removeGhostHighlight();
    };
    const handleKeyDown = (event) => {
        if (event.code === "Escape") {
            removeGhostHighlight();
            setTip(null);
        }
    };
    const handleScaleValue = () => {
        if (viewerRef.current) {
            viewerRef.current.currentScaleValue = pdfScaleValue.toString();
        }
    };
    const renderHighlightLayers = () => {
        if (!viewerRef.current)
            return;
        for (let pageNumber = 1; pageNumber <= pdfDocument.numPages; pageNumber++) {
            const highlightBindings = highlightBindingsRef.current[pageNumber];
            // Need to check if container is still attached to the DOM as PDF.js can unload pages.
            if (highlightBindings?.container?.isConnected) {
                renderHighlightLayer(highlightBindings, pageNumber);
            }
            else {
                const { textLayer } = viewerRef.current.getPageView(pageNumber - 1) || {};
                if (!textLayer)
                    continue; // Viewer hasn't rendered page yet
                const highlightLayer = findOrCreateHighlightLayer(textLayer.div);
                if (highlightLayer) {
                    const reactRoot = createRoot(highlightLayer);
                    highlightBindingsRef.current[pageNumber] = {
                        reactRoot,
                        container: highlightLayer,
                        textLayer: textLayer.div,
                    };
                    renderHighlightLayer(highlightBindingsRef.current[pageNumber], pageNumber);
                }
            }
        }
    };
    const renderHighlightLayer = (highlightBindings, pageNumber) => {
        if (!viewerRef.current)
            return;
        highlightBindings.reactRoot.render(React.createElement(TipViewerContext.Provider, { value: tipViewerUtils },
            React.createElement(HighlightLayer, { highlightsByPage: groupHighlightsByPage([
                    ...highlights,
                    ghostHighlightRef.current,
                ]), pageNumber: pageNumber, isSelectionInProgress: isSelectionInProgress, scrolledToHighlightId: scrolledToHighlightIdRef.current, viewer: viewerRef.current, highlightBindings: highlightBindings, children: children })));
    };
    const tipViewerUtils = {
        currentTip: tip,
        setTip: (tip) => setTip(tip),
        toggleEditInProgress: (flag) => {
            if (flag !== undefined) {
                isEditInProgressRef.current = flag;
            }
            else {
                isEditInProgressRef.current = !isEditInProgressRef.current;
            }
            disableTextSelection(viewerRef.current, isEditInProgressRef.current);
        },
        isEditInProgress: () => isEditInProgressRef.current,
    };
    scrollRef && scrollRef(scrollTo);
    tipViewerUtilsRef && tipViewerUtilsRef(tipViewerUtils);
    pdfViewerRef && viewerRef.current && pdfViewerRef(viewerRef.current);
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { ref: containerNodeRef, className: "PdfHighlighter", onPointerDown: handleMouseDown, style: style },
            React.createElement("div", { className: "pdfViewer" }),
            React.createElement("style", null, `
          .textLayer ::selection {
            background: ${textSelectionColor};
          }
        `),
            React.createElement(TipViewerContext.Provider, { value: tipViewerUtils },
                isViewerReady && React.createElement(TipRenderer, { viewer: viewerRef.current }),
                isViewerReady && enableAreaSelection && (React.createElement(MouseSelectionRenderer, { viewer: viewerRef.current, onChange: (isVisible) => (isAreaSelectionInProgressRef.current = isVisible), enableAreaSelection: enableAreaSelection, style: mouseSelectionStyle, onAfterSelection: (viewportPosition, scaledPosition, image, resetSelection) => {
                        const selectionUtils = {
                            selectionPosition: scaledPosition,
                            selectionContent: { image },
                            removeGhostHighlight,
                            makeGhostHighlight: () => {
                                ghostHighlightRef.current = {
                                    position: scaledPosition,
                                    content: { image },
                                };
                                resetSelection();
                                renderHighlightLayers();
                            },
                        };
                        onSelectionFinished && onSelectionFinished(selectionUtils);
                        if (selectionTip) {
                            const newTip = {
                                position: viewportPosition,
                                content: (React.createElement(SelectionContext.Provider, { value: selectionUtils }, selectionTip)),
                            };
                            setTip(newTip);
                        }
                    } }))))));
};
export default PdfHighlighter;
//# sourceMappingURL=PdfHighlighter.js.map